package de.duehl.vocabulary.japanese.logic.symbol.kanji.test;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.duehl.basics.text.Text;
import de.duehl.math.stochastic.RandomSample;
import de.duehl.math.stochastic.RandomSampleWithoutPutBack;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.common.persistence.Options;
import de.duehl.vocabulary.japanese.data.KanjiSet;
import de.duehl.vocabulary.japanese.data.symbol.Kanji;
import de.duehl.vocabulary.japanese.logic.symbol.kanji.internal.InternalKanjiDataRequester;
import de.duehl.vocabulary.japanese.logic.symbol.kanji.test.data.KanjiForTestSelectionMethod;
import de.duehl.vocabulary.japanese.ui.dialog.kanjitest.KanjiTestParameterChooser;
import de.duehl.vocabulary.japanese.ui.dialog.kanjitest.KanjiTester;

/**
 * Diese Klasse lässt zunächst auswählen, ob einige oder alle Kanji abgefragte werden sollen und ob
 * die Kanji in der originalen Reihenfolge oder zufällig sortiert abgefragt werden sollen.
 *
 * Anschließend fragt es ein Kanji nach dem anderen ab: Das Symbol wird angezeigt und der Benutzer
 * kann deutsche Bedeutungen sowie ON- und kun-Lesungen eingeben.
 *
 * Anhand der in den Kanji-Enum-Objekten wird dann überprüft, ob (ausreichend viel) eingegeben wurde
 * und im Dialog zur Bewertung der einen Abfrage werden alle Daten zu dem Kanji angezeigt.
 *
 * Bei der deutschen Übersetzung trenne ich die Benutzereingabe an allen Kombinationen von
 * Leerzeichen und Kommata.
 *
 * Das wird auch bei den Lesungen gemacht, dort wird aber auch noch auf eckige Klammern geachtet.
 *
 * @version 1.01     2024-11-21
 * @author Christian Dühl
 */

public class KanjiTesterLogic {

    private static final boolean DEBUG = false;


    /** Die Programmoptionen. */
    private final Options options;

    /** Das Objekt, das zu einem Kanji die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalKanjiDataRequester requester;

    /** Die Liste der benutzerdefinierten Kanji-Mengen. */
    private final List<KanjiSet> kanjiSets;

    /** Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private final Point parentLocation;

    /** Das anzuzeigende ProgrammIcon. */
    private final Image programImage;

    /** Gibt an, ob man fortsetzen soll. */
    private boolean goAhead;

    /** Die Liste an Kanji, aus denen ausgewählt wird. */
    private List<Kanji> kanjiList;

    /** Die Anzahl der abzufragenden Kanji. */
    private int numberOfKanjiToTest;

    /** Gibt an ob die abzufragenden Kanji in der originalen Reihenfolge belassen werden sollen. */
    private boolean useOriginalKanjiOrder;

    /** Die Liste der abzufragenden Kanji. */
    private List<Kanji> kanjiToTest;

    /** Die Art wie Kanji für die Abfrage ausgewählt werden. */
    private KanjiForTestSelectionMethod selectionMethod;

    /**
     * Gibt an, ob bei der Überprüfung der deutschen Bedeutung auf Groß-/Kleinschreibung geachtet
     * werden soll.
     */
    private boolean germanMeaningCaseSensitivity;

    /**
     * Gibt an, ob bei der Überprüfung der ON-Lesung auf Groß-/Kleinschreibung geachtet werden
     * soll.
     */
    private boolean onLesungCaseSensitivity;

    /**
     * Gibt an, ob bei der Überprüfung der kun-Lesung auf Groß-/Kleinschreibung geachtet werden
     * soll.
     */
    private boolean kunLesungCaseSensitivity;

    /**
     * Konstruktor.
     *
     * @param options
     *            Die Programmoptionen.
     * @param requester
     *            Das Objekt, das zu einem Kanji die internen, benutzerabhängigen Daten abrufen
     *            kann.
     * @param kanjiSets
     *            Die Liste der benutzerdefinierten Kanji-Mengen.
     * @param parentLocation
     *            Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Das anzuzeigende ProgrammIcon.
     */
    public KanjiTesterLogic(Options options, InternalKanjiDataRequester requester,
            List<KanjiSet> kanjiSets, Point parentLocation, Image programImage) {
        this.options = options;
        this.requester = requester;
        this.kanjiSets = kanjiSets;
        this.parentLocation = parentLocation;
        this.programImage = programImage;
    }

    /** Führt den Test durch. */
    public void test() {
        init();

        if (goAhead) askTestParameters();
        if (goAhead) determineKanjiToTest();
        if (goAhead) startRealTest();
    }

    private void init() {
        goAhead = true;
    }

    private void askTestParameters() {
        KanjiTestParameterChooser chooser = new KanjiTestParameterChooser(kanjiSets, options,
                parentLocation, programImage);
        chooser.setVisible(true);
        if (chooser.isApplied()) {
            kanjiList = chooser.getKanjiList();
            numberOfKanjiToTest = chooser.getNumberOfKanjiToTest();
            useOriginalKanjiOrder = chooser.isUseOriginalKanjiOrder();
            selectionMethod = chooser.getSelectionMethod();
            germanMeaningCaseSensitivity = chooser.isGermanMeaningCaseSensitivity();
            onLesungCaseSensitivity = chooser.isOnLesungCaseSensitivity();
            kunLesungCaseSensitivity = chooser.isKunLesungCaseSensitivity();
            if (DEBUG) Text.say("numberOfKanjiToTest  = " + numberOfKanjiToTest);
            if (DEBUG) Text.say("sortKanjiToTest      = " + useOriginalKanjiOrder);

            if (kanjiList.isEmpty()) {
                String title = "Leer Menge an Kanji ausgewählt";
                String message = "Sie haben eine leere Menge an abzufragenden Kanji ausgewählt, "
                        + "daher ist nichts abzufragen.";
                GuiTools.informUser(title, message);
                goAhead = false;
            }
        }
        else {
            goAhead = false;
        }
    }

    private void determineKanjiToTest() {
        switch (selectionMethod) {
            case RANDOM_BY_NUMBER:
                determineKanjiToTestByRandomNumber();
                return;
            case ALL:
                useAllKanji();
                return;
            case LAST_N:
                useLastNKanji();
                return;
            case UNKNOWN:
            default:
                goAhead = false;
        }
    }

    private void determineKanjiToTestByRandomNumber() {
        List<Integer> kanjiPositionsSample = createKanjiPositionSample();
        fillKanjiToTestWithKanjiPositionSample(kanjiPositionsSample);
    }

    /**
     * Erstellt eine zufällige Menge von Positionen zwischen 1 und der Anzahl der bekannten Kanji
     * ohne zurücklegen.
     *
     * Falls die originale Reihenfolge benutzt werden soll, werden die Positionen sortiert.
     */
    private List<Integer> createKanjiPositionSample() {
        int from = 1;
        int to = kanjiList.size();
        int sampleSize = Math.min(kanjiList.size(), numberOfKanjiToTest);

        RandomSample randomSample = new RandomSampleWithoutPutBack(from, to, sampleSize);
        randomSample.drawSample();
        List<Integer> kanjiPositionsSample = randomSample.getSample();

        if (useOriginalKanjiOrder) {
            Collections.sort(kanjiPositionsSample);
        }

        if (DEBUG) Text.say("kanjiPositionsSample = " + kanjiPositionsSample);

        return kanjiPositionsSample;
    }

    /**
     * Erstellt aus der Positionen zwischen 1 und der Anzahl der bekannten Kanji die entsprechende
     * Liste mit den Kanji.
     *
     * @param kanjiPositionsSample
     *            Liste mit den Positionen zwischen 1 und der Anzahl der bekannten Kanji.
     */
    private void fillKanjiToTestWithKanjiPositionSample(List<Integer> kanjiPositionsSample) {
        kanjiToTest = new ArrayList<>();

        for (int kanjiPosition : kanjiPositionsSample) {
            int kanjiIndex = kanjiPosition - 1;
            Kanji kanji = kanjiList.get(kanjiIndex);
            kanjiToTest.add(kanji);
        }
    }

    private void useAllKanji() {
        kanjiToTest = new ArrayList<>();
        kanjiToTest.addAll(kanjiList);

        if (!useOriginalKanjiOrder) {
            Collections.shuffle(kanjiToTest);
        }
    }

    private void useLastNKanji() {
        kanjiToTest = new ArrayList<>();

        int size = kanjiList.size();

        int startIndex = Math.max(0, size - numberOfKanjiToTest);

        for (int index = startIndex; index <= size - 1; ++index) {
            Kanji kanji = kanjiList.get(index);
            kanjiToTest.add(kanji);
        }

        if (!useOriginalKanjiOrder) {
            Collections.shuffle(kanjiToTest);
        }
    }

    private void startRealTest() {
        KanjiTester tester = new KanjiTester(kanjiToTest, germanMeaningCaseSensitivity,
                onLesungCaseSensitivity, kunLesungCaseSensitivity, options, requester,
                parentLocation, programImage);
        tester.setVisible(true);
    }

}
