package de.duehl.vocabulary.japanese.ui.components.bars;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Point;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.buttons.painted.DownButton;
import de.duehl.swing.ui.buttons.painted.FirstButton;
import de.duehl.swing.ui.buttons.painted.LastButton;
import de.duehl.swing.ui.buttons.painted.UpButton;
import de.duehl.swing.ui.colors.ColorTool;
import de.duehl.vocabulary.japanese.data.symbol.Kanji;
import de.duehl.vocabulary.japanese.logic.symbol.kanji.internal.InternalKanjiDataRequester;
import de.duehl.vocabulary.japanese.ui.dialog.kanjiset.KanjiSetEditor;
import de.duehl.vocabulary.japanese.ui.dialog.kanjitest.components.KanjiDetailDialog;

import static de.duehl.vocabulary.japanese.ui.components.bars.VocabularyBar.BUTTON_DIMENSION;
import static de.duehl.vocabulary.japanese.ui.components.bars.VocabularyBar.DISABLED_COLOR;
import static de.duehl.vocabulary.japanese.ui.components.bars.VocabularyBar.MOVE_BUTTON_COLOR;

/**
 * Diese Klasse stellt die grafische Oberfläche der Anzeige eines Kanji im Editor zur Bearbeiten
 * von Kanji-Mengen dar.
 *
 * @version 1.01     2024-11-22
 * @author Christian Dühl
 */

public class KanjiBar {

    /** Das angezeigte Kanji. */
    private final Kanji kanji;

    /** Wird ausgeführt, wenn der Button gedrückt wird. */
    private final Runnable buttonAction;

    /** Der Dialog, in dem die Bars bewegt etc. werden. */
    private final KanjiSetEditor gui;

    /** Das Objekt, das zu einem Kanji die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalKanjiDataRequester kanjiRequester;

    /** Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private final Point parentLocation;

    /** Das Icon für das Programm. */
    private final Image programImage;

    /** Der Panel auf dem das Kanji dargestellt wird. */
    private final JPanel panel;

    /** Der Panel auf dem der Schalter und vielleicht auch die CheckBox dargestellt wird. */
    private final JPanel rightPanel;

    /** Das Label, auf dem das Kanji dargestellt wird. */
    private final JLabel kanjiLabel;

    /** CheckBox zur Selektion. */
    private final JCheckBox selectionCheckBox;

    /** Der Button zum Anzeigen der Details des Kanji. */
    private final JButton detailsButton;

    /** Der Multifunktions-Button (Hinzufügen zur Liste oder Löschen aus Liste). */
    private final JButton multifunctionButton;

    /** Der Panel rechts mit Buttons zum Verschieben. */
    private final JPanel moveButtonsPanel;

    /** Der Schalter um das Kanji in der Kanji-Menge nach oben zu verschieben. */
    private final UpButton upButton;

    /** Der Schalter um das Kanji in der Kanji-Menge nach unten zu verschieben. */
    private final DownButton downButton;

    /** Gibt an, ob die Schalter zum Bewegen der Bar genutzt werden. */
    private boolean canBeMooved;

    /**
     * Konstruktor für den Dialog zur Bearbeitung einer Kanji-Menge (KanjiSetEditor) (rechte oder
     * linke Seite).
     *
     * Diese Klasse wird sowohl in der Kanji-Menge rechts als auch links in der gefilterten Liste
     * aller Kanji genutzt.
     *
     * @param kanji
     *            Das angezeigte Kanji.
     * @param buttonAction
     *            Wird ausgeführt, wenn der Button gedrückt wird.
     * @param gui
     *            Der Dialog, in dem die Bars bewegt etc. werden.
     * @param kanjiRequester
     *            Das Objekt, das zu einem Kanji die internen, benutzerabhängigen Daten abrufen
     *            kann.
     * @param parentLocation
     *            Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Das Icon für das Programm.
     */
    public KanjiBar(Kanji kanji, Runnable buttonAction,
            KanjiSetEditor gui, InternalKanjiDataRequester kanjiRequester, Point parentLocation,
            Image programImage) {
        this.kanji = kanji;
        this.buttonAction = buttonAction;
        this.gui = gui;
        this.kanjiRequester = kanjiRequester;
        this.parentLocation = parentLocation;
        this.programImage = programImage;

        panel = new JPanel();
        kanjiLabel = new JLabel();
        rightPanel = new JPanel();
        detailsButton = new JButton();
        multifunctionButton = new JButton();
        selectionCheckBox = new JCheckBox();
        moveButtonsPanel = new JPanel();

        upButton = new UpButton(ColorTool.getColorByName(DISABLED_COLOR));
        downButton = new DownButton(ColorTool.getColorByName(DISABLED_COLOR));

        init();
    }

    private void init() {
        initKanjiLabel();
        initButtons();
        initRightPanel();
    }

    private void initKanjiLabel() {
        kanjiLabel.setText(createKanjiText());
        GuiTools.biggerFont(kanjiLabel, 20);
        kanjiLabel.setHorizontalAlignment(JLabel.CENTER);
    }

    private void initButtons() {
        initMultifunctionButton();
        initDetailsButton();
    }

    private void initMultifunctionButton() {
        multifunctionButton.addActionListener(e -> buttonAction.run());
    }

    private void initDetailsButton() {
        detailsButton.setText("?");
        detailsButton.addActionListener(e -> showKanjiDetails());
    }

    private void showKanjiDetails() {
        KanjiDetailDialog dialog = new KanjiDetailDialog(kanji, kanjiRequester, parentLocation,
                programImage);
        dialog.setVisible(true);
    }

    private void initRightPanel() {
        rightPanel.setLayout(new BorderLayout());
        rightPanel.add(multifunctionButton, BorderLayout.EAST);
    }

    /**
     * Nutzt den Button im Dialog zur Bearbeitung von Kanji-Mengen auf der linken Seite zum
     * Hinzufügen und zeigt auch die CheckBox zur Auswahl der Kanji an.
     */
    public void useButtonAsAddKanjiToList() {
        multifunctionButton.setText("hinzufügen");
        rightPanel.add(selectionCheckBox, BorderLayout.WEST);
        canBeMooved = true;
    }

    /**
     * Nutzt den Button im Dialog zur Bearbeitung von Kanji-Mengen auf der rechten Seite zum Löschen.
     * Die CheckBox wird nicht angezeigt.
     */
    public void useButtonAsDeletion() {
        multifunctionButton.setText("entfernen");
        canBeMooved = true;
    }

    /** Baut die Gui auf. */
    public void createGui() {
        initDependingOfCanBeMoved();
        fillPanel();
    }

    /** Kennzeichnet das Kanji als ausgegraut (es ist bereits in der Liste). */
    public void disable() {
        detailsButton.setEnabled(false);
        multifunctionButton.setEnabled(false);
        selectionCheckBox.setSelected(false);
        selectionCheckBox.setEnabled(false);
        kanjiLabel.setForeground(Color.LIGHT_GRAY);
    }

    /** Kennzeichnet das Kanji als aktiv (es ist nicht in der Liste). */
    public void enable() {
        detailsButton.setEnabled(true);
        multifunctionButton.setEnabled(true);
        selectionCheckBox.setEnabled(true);
        kanjiLabel.setForeground(Color.BLACK);
    }

    private void initDependingOfCanBeMoved() {
        if (canBeMooved) {
            initShowOrHidePanels();
            initUpAndDownButtons();
        }
    }

    private void initShowOrHidePanels() {
        moveButtonsPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 0));
    }

    private void initUpAndDownButtons() {
        upButton.forceSquare();
        upButton.setPreferredSize(BUTTON_DIMENSION);
        upButton.addActionListener(e -> gui.moveBarUp(kanji));

        downButton.forceSquare();
        downButton.setPreferredSize(BUTTON_DIMENSION);
        downButton.addActionListener(e -> gui.moveBarDown(kanji));

        setUpAndDownButtonColorsAndEnabled();
    }

    private void populateMoveButtonsPanel(boolean showDeleteAndExportButtonsOnBars) {
        moveButtonsPanel.removeAll();

        if (showDeleteAndExportButtonsOnBars) {
            moveButtonsPanel.add(createMoveToTopButton());
            moveButtonsPanel.add(upButton);
            moveButtonsPanel.add(downButton);
            moveButtonsPanel.add(createMoveToBottomButton());
        }
    }

    private Component createMoveToTopButton() {
        FirstButton button = new FirstButton(ColorTool.getColorByName(MOVE_BUTTON_COLOR));
        button.setPreferredSize(BUTTON_DIMENSION);
        button.setHorizontal(false);
        button.addActionListener(e -> gui.moveBarToFirst(kanji));
        return button;
    }

    private Component createMoveToBottomButton() {
        LastButton button = new LastButton(ColorTool.getColorByName(MOVE_BUTTON_COLOR));
        button.setPreferredSize(BUTTON_DIMENSION);
        button.setHorizontal(false);
        button.addActionListener(e -> gui.moveBarToLast(kanji));
        return button;
    }

    public void setUpAndDownButtonColorsAndEnabled() {
        boolean canMoveUp = gui.canBarMoveUp(kanji);
        Color upButtonColor = canMoveUp
                ? ColorTool.getColorByName(MOVE_BUTTON_COLOR)
                : ColorTool.getColorByName(DISABLED_COLOR);
        upButton.setForegroundColor(upButtonColor);
        upButton.setEnabled(canMoveUp);
        upButton.repaint();

        boolean canMoveDown = gui.canBarMoveDown(kanji);
        Color downButtonColor = canMoveDown
                ? ColorTool.getColorByName(MOVE_BUTTON_COLOR)
                : ColorTool.getColorByName(DISABLED_COLOR);
        downButton.setForegroundColor(downButtonColor);
        downButton.setEnabled(canMoveDown);
        downButton.repaint();
    }

    private void fillPanel() {
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(detailsButton, BorderLayout.WEST);
        panel.add(kanjiLabel, BorderLayout.CENTER);
        panel.add(createButtonPart(), BorderLayout.EAST);
    }

    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(rightPanel, BorderLayout.WEST);
        panel.add(moveButtonsPanel, BorderLayout.EAST);

        return panel;
    }

    private String createKanjiText() {
        return kanji.getCharacter();
    }

    /** Getter für den Panel auf dem das Kanji dargestellt wird. */
    public Component getPanel() {
        return panel;
    }

    /** Setzt das Kanji aus aus- oder nicht ausgewählt. */
    public void setSelected(boolean selected) {
        selectionCheckBox.setSelected(selected);
    }

    /** Gibt an, ob das Kanji ausgewählt ist. */
    public boolean isSelected() {
        return selectionCheckBox.isSelected();
    }

    /** Zeigt die Buttons zum Verschieben an oder blendet wie aus. */
    public void showMoveButtonsOnBars(boolean showMoveButtonsOnBars) {
        populateMoveButtonsPanel(showMoveButtonsOnBars);
    }

}
